#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){

    mainText = R"DONE(Some things are hard to write about. Take soil,
for instance. Soil, Oxford dictionary reads, “is the
upper layer of earth in which plants grow, a black or
dark brown material typically consisting of a mixture
of organic remains, clay, and rock particles”.
I wonder why I have chosen to write
about soil, as I don’t seem to have had many
encounters with it. Perhaps because of that?
Intuiting its importance, but never minding it?
So far it has appeared rather distant. Absent
from my thoughts and words. But now, when
I think of it, I see a charismatic substance.
Soil, according to Merriam-Webster, is
“1. firm land: earth;
 2. a) the upper layer of earth that may be dug
       or plowed and in which plants grow; and
    b) the superficial unconsolidated and
       usually weathered part of the mantle of
       a planet and especially of the earth”.

The Soil Science Society of America defines soil
as “a mixture of minerals, dead and living
organisms (organic materials), air, and water.”
The “soil” entry in Encyclopaedia Britannica starts
with “soil is the biologically active,
porous medium […] serving as a reservoir of
water and nutrients, as a medium for the filtration
and breakdown of injurious wastes, and as
a participant in the cycling of carbon and other
elements through the global ecosystem.”
Such nebulous definitions, though, aren’t
most definitions unsettled? (all definitions are
blasphemy!)… like “soil is (a mix of) all and nothing,
doing anything and everything, everywhere”. How
one defines or translates soil by posing a question
always affects the answer. Soil can mean earth,
ground, dirt, clay, turf, humus, silt, loam, land,
clod, terra, territory, landscape, country, a political
power base, an aspect of divinity, a terrain
to cultivate, or a resource to be exploited… This
is going to be difficult. I don’t know yet where to
start. I hope to capture something essential.)DONE";

    shaderCatalog = std::make_unique <ShaderCatalog>("data/ofxGPUFont/shaders/GL3");
    backgroundShader = shaderCatalog->get("background");
    fontShader = shaderCatalog->get("font");

    {
        FT_Error error = FT_Init_FreeType(&library);
        if(error){
            std::cerr << "ERROR: failed to initialize FreeType" << std::endl;
            ofExit();
        }
    }

    currentFontPath = "data/celines-fonts/Version-1-var.ttf";
    tryUpdateMainFont(library,
                      currentFontPath,
                      mainText,
                      font,
                      bb);

    transform = Transform();
}

//--------------------------------------------------------------
void ofApp::update(){

}

//--------------------------------------------------------------
void ofApp::draw(){
    GLuint location;

    int width = ofGetWidth();
    int height = ofGetHeight();

    glm::mat4 projection = transform.getProjectionMatrix((float)width / height);
    glm::mat4 view = transform.getViewMatrix();
    glm::mat4 model = glm::mat4(1.0f);

    {     // Draw background.
        GLuint program = backgroundShader->program;
        glUseProgram(program);
        glBindVertexArray(emptyVAO);
        glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
        glBindVertexArray(0);
        glUseProgram(0);
    }

    // Uses premultiplied-alpha.
    glEnable(GL_BLEND);
    glBlendEquation(GL_FUNC_ADD);
    glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);

    if(font){
        GLuint program = fontShader->program;
        glUseProgram(program);

        font->program = program;
        font->drawSetup();

        location = glGetUniformLocation(program, "projection");
        glUniformMatrix4fv(location, 1, false, glm::value_ptr(projection));
        location = glGetUniformLocation(program, "view");
        glUniformMatrix4fv(location, 1, false, glm::value_ptr(view));
        location = glGetUniformLocation(program, "model");
        glUniformMatrix4fv(location, 1, false, glm::value_ptr(model));
        float z = 0;
        location = glGetUniformLocation(program, "z");
        glUniform1f(location, z);

        location = glGetUniformLocation(program, "color");
        glUniform4f(location, 1.0f, 1.0f, 1.0f, 1.0f);

        location = glGetUniformLocation(program, "antiAliasingWindowSize");
        glUniform1f(location, (float)antiAliasingWindowSize);
        location = glGetUniformLocation(program, "enableSuperSamplingAntiAliasing");
        glUniform1i(location, enableSuperSamplingAntiAliasing);
        location = glGetUniformLocation(program, "enableControlPointsVisualization");
        glUniform1i(location, enableControlPointsVisualization);

        float cx = 0.5f * (bb.minX + bb.maxX);
        float cy = 0.5f * (bb.minY + bb.maxY);
        font->draw(-cx, -cy, 0, mainText);
        glUseProgram(0);
    }

    glDisable(GL_BLEND);

}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){

}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){

}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y){

}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseEntered(int x, int y){

}

//--------------------------------------------------------------
void ofApp::mouseExited(int x, int y){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){

}
